@@ -24,6 +24,7 @@
 #include <unistd.h>
 
 #include "xchat.h"
+#include <glib/ghash.h>
 
 #include "cfgfiles.h"
 #include "fe.h"
@@ -47,8 +48,6 @@ static const struct defaultserver def[] =
 {
 	{"2600net",	0},
 	{0,			"irc.2600.net"},
-	{0,			"ptolomea.2600.net"},
-	{0,			"collective.2600.net"},
 
 	{"AccessIRC",	0},
 	{0,			"irc.accessirc.net"},
@@ -120,6 +119,9 @@ static const struct defaultserver def[] =
 	{0,			"irc.azzurra.org"},
 	{0,			"crypto.azzurra.org"},
 
+	{"Beirut", 0},
+	{0,			"irc.beirut.com"},
+
 	{"Buzzpot", 0},
 	{0,			"irc.chatspike.net"},
 
@@ -194,7 +196,6 @@ static const struct defaultserver def[] =
 
 	{"FEFNet",	0},
 	{0,			"irc.fef.net"},
-	{0,			"irc.villagenet.com"},
 	{0,			"irc.ggn.net"},
 	{0,			"irc.vendetta.com"},
 
@@ -234,6 +235,9 @@ static const struct defaultserver def[] =
 	{"Hashmark",	0},
 	{0,			"irc.hashmark.net"},
 
+	{"IdleMonkeys", 0},
+	{0,			"irc.idlemonkeys.net"},
+
 	{"Infinity-IRC",	0},
 	{0,			"Atlanta.GA.US.Infinity-IRC.Org"},
 	{0,			"Babylon.NY.US.Infinity-IRC.Org"},
@@ -268,6 +272,9 @@ static const struct defaultserver def[] =
 	{0,			"irc.canadian.net"},
 	{0,			"irc.irctoo.net"},
 
+	{"Krstarica", 0},
+	{0,			"irc.krstarica.com"},
+
 	{"Librenet",	0},
 	{0,			"irc.librenet.net"},
 	{0,			"ielf.fr.librenet.net"},
@@ -286,6 +293,9 @@ static const struct defaultserver def[] =
 	{"Majistic",	0},
 	{0,			"irc.majistic.net"},
 
+	{"MIXXnet",		0},
+	{0,			"irc.mixxnet.net"},
+
 	{"NeverNET",	0},
 	{0,			"irc.nevernet.net"},
 	{0,			"imagine.nevernet.net"},
@@ -309,6 +319,9 @@ static const struct defaultserver def[] =
 	{"NullusNet",	0},
 	{0,			"irc.nullus.net"},
 
+	{"Oceanius", 0},
+	{0,			"irc.oceanius.com"},
+
 	{"OFTC",	0},
 	{0,			"irc.oftc.net"},
 
@@ -385,7 +398,7 @@ static const struct defaultserver def[] =
 	{0,			"lisa.rizenet.org"},
 	{0,			"scott.rizenet.org"},
 
-	{"RusNet", 0, 0, "KOI8-R"},
+	{"RusNet", 0, 0, "KOI8-R (Cyrillic)"},
 	{0,			"irc.tomsk.net"},
 	{0,			"irc.rinet.ru"},
 	{0,			"irc.run.net"},
@@ -428,9 +441,6 @@ static const struct defaultserver def[] =
 	{"TNI3",			0},
 	{0,			"irc.tni3.com"},
 
-	{"TopGamers",	0},
-	{0,			"irc.topgamers.net"},
-
 	{"UnderNet",	0},
 	{0,			"us.undernet.org"},
 	{0,			"eu.undernet.org"},
@@ -443,6 +453,9 @@ static const struct defaultserver def[] =
 	{0,			"irc.telecoms.bg"},
 	{0,			"irc.tu-varna.edu"},
 
+	{"Whiffle",	0},
+	{0,			"irc.whiffle.org"},
+
 	{"Worldnet",		0},
 	{0,			"irc.worldnet.net"},
 	{0,			"irc.fr.worldnet.net"},
@@ -469,7 +482,7 @@ GSList *network_list = 0;
 
 
 void
-servlist_connect (session *sess, ircnet *net)
+servlist_connect (session *sess, ircnet *net, gboolean join)
 {
 	ircserver *ircserv;
 	GSList *list;
@@ -490,12 +503,16 @@ servlist_connect (session *sess, ircnet *net)
 	/* incase a protocol switch is added to the servlist gui */
 	server_fill_her_up (sess->server);
 
-	sess->willjoinchannel[0] = 0;
-	serv->password[0] = 0;
+	if (join)
+	{
+		sess->willjoinchannel[0] = 0;
 
-	if (net->autojoin)
-		safe_strcpy (sess->willjoinchannel, net->autojoin,
-						 sizeof (sess->willjoinchannel));
+		if (net->autojoin)
+			safe_strcpy (sess->willjoinchannel, net->autojoin,
+							 sizeof (sess->willjoinchannel));
+	}
+
+	serv->password[0] = 0;
 	if (net->pass)
 		safe_strcpy (serv->password, net->pass, sizeof (serv->password));
 
@@ -519,6 +536,9 @@ servlist_connect (session *sess, ircnet *net)
 	serv->network = net;
 
 	port = strrchr (ircserv->hostname, '/');
+	if (!port)
+	    port = strchr (ircserv->hostname, ':');
+	
 	if (port)
 	{
 		*port = 0;
@@ -543,7 +563,7 @@ servlist_connect (session *sess, ircnet *net)
 }
 
 int
-servlist_connect_by_netname (session *sess, char *network)
+servlist_connect_by_netname (session *sess, char *network, gboolean join)
 {
 	ircnet *net;
 	GSList *list = network_list;
@@ -554,7 +574,7 @@ servlist_connect_by_netname (session *sess, char *network)
 
 		if (strcasecmp (net->name, network) == 0)
 		{
-			servlist_connect (sess, net);
+			servlist_connect (sess, net, join);
 			return 1;
 		}
 
@@ -597,7 +617,7 @@ servlist_auto_connect (session *sess)
 
 		if (net->flags & FLAG_AUTO_CONNECT)
 		{
-			servlist_connect (sess, net);
+			servlist_connect (sess, net, TRUE);
 			ret = 1;
 		}
 
@@ -612,7 +632,7 @@ servlist_cycle_cb (server *serv)
 {
 	PrintTextf (serv->server_session,
 		_("Cycling to next server in %s...\n"), ((ircnet *)serv->network)->name);
-	servlist_connect (serv->server_session, serv->network);
+	servlist_connect (serv->server_session, serv->network, TRUE);
 
 	return 0;
 }
@@ -644,7 +664,7 @@ servlist_cycle (server *serv)
 			if (del)
 				serv->recondelay_tag = fe_timeout_add (del, servlist_cycle_cb, serv);
 			else
-				servlist_connect (serv->server_session, net);
+				servlist_connect (serv->server_session, net, TRUE);
 
 			return TRUE;
 		}
@@ -676,8 +696,38 @@ servlist_server_find (ircnet *net, char *name, int *pos)
 	return NULL;
 }
 
+/* find a network (e.g. (ircnet *) to "FreeNode") from a hostname
+   (e.g. "irc.eu.freenode.net") */
+
 ircnet *
-servlist_net_find (char *name, int *pos)
+servlist_net_find_from_server (char *server_name)
+{
+	GSList *list = network_list;
+	GSList *slist;
+	ircnet *net;
+	ircserver *serv;
+
+	while (list)
+	{
+		net = list->data;
+
+		slist = net->servlist;
+		while (slist)
+		{
+			serv = slist->data;
+			if (strcasecmp (serv->hostname, server_name) == 0)
+				return net;
+			slist = slist->next;
+		}
+
+		list = list->next;
+	}
+
+	return NULL;
+}
+
+ircnet *
+servlist_net_find (char *name, int *pos, int (*cmpfunc) (const char *, const char *))
 {
 	GSList *list = network_list;
 	ircnet *net;
@@ -686,7 +736,7 @@ servlist_net_find (char *name, int *pos)
 	while (list)
 	{
 		net = list->data;
-		if (strcmp (net->name, name) == 0)
+		if (cmpfunc (net->name, name) == 0)
 		{
 			if (pos)
 				*pos = i;
@@ -744,6 +794,8 @@ servlist_net_remove (ircnet *net)
 
 	if (net->nick)
 		free (net->nick);
+	if (net->nick2)
+		free (net->nick2);
 	if (net->user)
 		free (net->user);
 	if (net->real)
@@ -816,7 +868,7 @@ servlist_load_defaults (void)
 				free (net->encoding);
 				net->encoding = strdup (def[i].charset);
 			}
-			if (!strcmp (def[i].network, "ChatJunkies"))
+			if (g_str_hash (def[i].network) == 0x8e1b96f7)
 				prefs.slist_select = j;
 			j++;
 		} else
@@ -855,6 +907,9 @@ servlist_load (void)
 			case 'I':
 				net->nick = strdup (buf + 2);
 				break;
+			case 'i':
+				net->nick2 = strdup (buf + 2);
+				break;
 			case 'U':
 				net->user = strdup (buf + 2);
 				break;
@@ -977,6 +1032,8 @@ servlist_save (void)
 		fprintf (fp, "N=%s\n", net->name);
 		if (net->nick)
 			fprintf (fp, "I=%s\n", net->nick);
+		if (net->nick2)
+			fprintf (fp, "i=%s\n", net->nick2);
 		if (net->user)
 			fprintf (fp, "U=%s\n", net->user);
 		if (net->real)
@@ -995,7 +1052,7 @@ servlist_save (void)
 			{
 				snprintf (buf, sizeof (buf), _("Warning: \"%s\" character set is unknown. No conversion will be applied for network %s."),
 							 net->encoding, net->name);
-				fe_message (buf, FALSE);
+				fe_message (buf, FE_MSG_WARN);
 			}
 		}
 
