@@ -66,117 +66,201 @@ log_close (session *sess)
 }
 
 static void
-log_create_filename (char *buf, char *servname, char *channame, char *netname)
+mkdir_p (char *dir)	/* like "mkdir -p" from a shell, FS encoding */
 {
-	char fn[256];
-	char *tmp, *dir, *sep;
-	int pathlen=510, c=0, mbl;
+	char *start = dir;
 
-	if (!rfc_casecmp (channame, servname))
+	/* the whole thing already exists? */
+	if (access (dir, F_OK) == 0)
+		return;
+
+	while (*dir)
 	{
-		channame = g_strdup ("server");
-	} else
-	{
-		sep = tmp = strdup (channame);
-		while (*tmp)
-		{
-			mbl = g_utf8_skip[((unsigned char *)tmp)[0]];
-			if (mbl == 1)
-			{
-#ifndef WIN32
-				*tmp = rfc_tolower (*tmp);
-				if (*tmp == '/')
-#else
-				/* win32 can't handle filenames with \|/>< characters */
-				if (*tmp == '\\' || *tmp == '|' || *tmp == '/' ||
-					 *tmp == '>' || *tmp == '<')
-#endif
-					*tmp = '_';
-			}
-			tmp += mbl;
-		}
-
-		channame = g_filename_from_utf8 (sep, -1, 0, 0, 0);
-		free (sep);
-	}
-
-	snprintf (buf, 512, "%s/xchatlogs", get_xdir_fs ());
-	if (access (buf, F_OK) != 0)
 #ifdef WIN32
-		mkdir (buf);
+		if (dir != start && (*dir == '/' || *dir == '\\'))
 #else
-		mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
+		if (dir != start && *dir == '/')
 #endif
-	auto_insert (fn, sizeof (fn), prefs.logmask, NULL, NULL, "", channame, "", "", netname, servname);
-	g_free (channame);
-
-	snprintf (buf, 512, "%s/xchatlogs/%s", get_xdir_fs (), fn);
-
- 	/* The following code handles subdirectories in logpath and creates
- 	 * them, if they don't exist. Useful with logmasks like "%c/%y.log" in
- 	 * ~/.xchat/xchat.conf.
- 	 *                       -- patch by tvk <tvk@von-koch.com>
- 	 */
-
-	if (access (buf, F_OK) != 0)
-	{
-		snprintf (buf, 512, "%s/xchatlogs/", get_xdir_fs ());
-		pathlen -= strlen(buf);
- 
-		/* how many sub-directories do we have? */
-		sep = fn;
-		while (*sep++) if (*sep=='/') c++;
-			
-		if (c) {
- 			/* create each sub-directory */
-			sep = strdup(fn);
-			dir = strtok(sep, "/");
-		
-			while (c-- && dir != NULL) {
-				strncat (buf, dir, pathlen);
-				strcat (buf, "/");
-				pathlen -= strlen(dir);
- 		
-				if (access (buf, F_OK) != 0)
-#ifdef WIN32	
-					mkdir (buf);
-#else	
-					mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
-#endif	
-				dir = strtok(NULL, "/");
-			}
-
-			/* append the filename */
-			strncat (buf, strrchr(fn, '/')+1, pathlen);
-			free (sep);
-		} else {
-			strncat (buf, fn, pathlen);
+		{
+			*dir = 0;
+#ifdef WIN32
+			mkdir (start);
+#else
+			mkdir (start, S_IRUSR | S_IWUSR | S_IXUSR);
+#endif
+			*dir = '/';
 		}
-
+		dir++;
 	}
 }
 
+static char *
+log_create_filename (char *channame)
+{
+	char *tmp, *ret;
+	int mbl;
+
+	ret = tmp = strdup (channame);
+	while (*tmp)
+	{
+		mbl = g_utf8_skip[((unsigned char *)tmp)[0]];
+		if (mbl == 1)
+		{
+#ifndef WIN32
+			*tmp = rfc_tolower (*tmp);
+			if (*tmp == '/')
+#else
+			/* win32 can't handle filenames with \|/><:"*? characters */
+			if (*tmp == '\\' || *tmp == '|' || *tmp == '/' ||
+				 *tmp == '>'  || *tmp == '<' || *tmp == ':' ||
+				 *tmp == '\"' || *tmp == '*' || *tmp == '?')
+#endif
+				*tmp = '_';
+		}
+		tmp += mbl;
+	}
+
+	return ret;
+}
+
+/* like strcpy, but % turns into %% */
+
+static char *
+log_escape_strcpy (char *dest, char *src, char *end)
+{
+	while (*src)
+	{
+		*dest = *src;
+		if (dest + 1 == end)
+			break;
+		dest++;
+		src++;
+
+		if (*src == '%')
+		{
+			if (dest + 1 == end)
+				break;
+			dest[0] = '%';
+			dest++;
+		}
+	}
+
+	dest[0] = 0;
+	return dest - 1;
+}
+
+/* substitutes %c %n %s into buffer */
+
+static void
+log_insert_vars (char *buf, int bufsize, char *fmt, char *c, char *n, char *s)
+{
+	char *end = buf + bufsize;
+
+	while (1)
+	{
+		switch (fmt[0])
+		{
+		case 0:
+			buf[0] = 0;
+			return;
+
+		case '%':
+			fmt++;
+			switch (fmt[0])
+			{
+			case 'c':
+				buf = log_escape_strcpy (buf, c, end);
+				break;
+			case 'n':
+				buf = log_escape_strcpy (buf, n, end);
+				break;
+			case 's':
+				buf = log_escape_strcpy (buf, s, end);
+				break;
+			default:
+				buf[0] = '%';
+				buf++;
+				buf[0] = fmt[0];
+				break;
+			}
+			break;
+
+		default:
+			buf[0] = fmt[0];
+		}
+		fmt++;
+		buf++;
+		/* doesn't fit? */
+		if (buf == end)
+		{
+			buf[-1] = 0;
+			return;
+		}
+	}
+}
+
+static char *
+log_create_pathname (char *servname, char *channame, char *netname)
+{
+	char fname[384];
+	char fnametime[384];
+	char *fs;
+	struct tm *tm;
+	time_t now;
+
+	if (!netname)
+		netname = "NETWORK";
+
+	/* first, everything is in UTF-8 */
+	if (!rfc_casecmp (channame, servname))
+		channame = strdup ("server");
+	else
+		channame = log_create_filename (channame);
+	log_insert_vars (fname, sizeof (fname), prefs.logmask, channame, netname, servname);
+	free (channame);
+
+	/* insert time/date */
+	now = time (NULL);
+	tm = localtime (&now);
+	strftime (fnametime, sizeof (fnametime), fname, tm);
+
+	/* create final path/filename */
+	snprintf (fname, sizeof (fname), "%s/xchatlogs/%s", get_xdir_utf8 (), fnametime);
+
+	/* now we need it in FileSystem encoding */
+	fs = g_filename_from_utf8 (fname, -1, 0, 0, 0);
+
+	/* create all the subdirectories */
+	if (fs)
+		mkdir_p (fs);
+
+	return fs;
+}
+
 static int
 log_open_file (char *servname, char *channame, char *netname)
 {
 	char buf[512];
 	int fd;
+	char *file;
 	time_t currenttime;
 
-	if (!netname)
-		netname = "NETWORK";
+	file = log_create_pathname (servname, channame, netname);
+	if (!file)
+		return -1;
 
-	log_create_filename (buf, servname, channame, netname);
 #ifdef WIN32
-	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY, S_IREAD|S_IWRITE);
+	fd = open (file, O_CREAT | O_APPEND | O_WRONLY, S_IREAD|S_IWRITE);
 #else
-	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY, 0644);
+	fd = open (file, O_CREAT | O_APPEND | O_WRONLY, 0644);
 #endif
+	g_free (file);
+
 	if (fd == -1)
 		return -1;
 	currenttime = time (NULL);
 	write (fd, buf,
-			 snprintf (buf, 510, _("**** BEGIN LOGGING AT %s\n"),
+			 snprintf (buf, sizeof (buf), _("**** BEGIN LOGGING AT %s\n"),
 						  ctime (&currenttime)));
 
 	return fd;
@@ -241,10 +325,25 @@ log_write (session *sess, char *text)
 {
 	char *temp;
 	char *stamp;
+	char *file;
 	int len;
 
 	if (sess->logfd != -1 && prefs.logging)
 	{
+		/* change to a different log file? */
+		file = log_create_pathname (sess->server->servername, sess->channel,
+											 server_get_network (sess->server, FALSE));
+		if (file)
+		{
+			if (access (file, F_OK) != 0)
+			{
+				close (sess->logfd);
+				sess->logfd = log_open_file (sess->server->servername, sess->channel,
+													  server_get_network (sess->server, FALSE));
+			}
+			g_free (file);
+		}
+
 		if (prefs.timestamp_logs)
 		{
 			len = get_stamp_str (prefs.timestamp_log_format, time (0), &stamp);
@@ -254,7 +353,7 @@ log_write (session *sess, char *text)
 				g_free (stamp);
 			}
 		}
-		temp = strip_color (text, -1, 1, 1);
+		temp = strip_color (text, -1, STRIP_ALL);
 		len = strlen (temp);
 		write (sess->logfd, temp, len);
 		/* lots of scripts/plugins print without a \n at the end */
@@ -1136,8 +1235,15 @@ load_text_events ()
 	pevent_make_pntevts ();
 }
 
+/*
+	CL: format_event now handles filtering of arguments:
+	1) if prefs.stripcolor is set, filter all style control codes from arguments
+	2) always strip \001 (ATTR_HIDDEN) from arguments: it is only for use in the format string itself
+*/
+#define ARG_FLAG(argn) (1 << argn)
+
 void
-format_event (session *sess, int index, char **args, char *o, int sizeofo)
+format_event (session *sess, int index, char **args, char *o, int sizeofo, unsigned int stripcolor_args)
 {
 	int len, oi, ii, numargs;
 	char *i, *ar, d, a, done_all = FALSE;
@@ -1184,8 +1290,8 @@ format_event (session *sess, int index, char **args, char *o, int sizeofo)
 				printf ("arg[%d] is NULL in print event\n", a + 1);
 			} else
 			{
-				len = strlen (ar);
-				memcpy (&o[oi], ar, len);
+				if (stripcolor_args & ARG_FLAG(a + 1)) len = strip_color2 (ar, -1, &o[oi], STRIP_ALL);
+				else len = strip_hidden_attribute (ar, &o[oi]);
 				oi += len;
 			}
 			break;
@@ -1217,10 +1323,10 @@ format_event (session *sess, int index, char **args, char *o, int sizeofo)
 }
 
 static void
-display_event (session *sess, int event, char **args)
+display_event (session *sess, int event, char **args, unsigned int stripcolor_args)
 {
 	char o[4096];
-	format_event (sess, event, args, o, sizeof (o));
+	format_event (sess, event, args, o, sizeof (o), stripcolor_args);
 	if (o[0])
 		PrintText (sess, o);
 }
@@ -1401,6 +1507,25 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 	return 0;
 }
 
+
+/* black n white(0/1) are bad colors for nicks, and we'll use color 2 for us */
+/* also light/dark gray (14/15) */
+/* 5,7,8 are all shades of yellow which happen to look dman near the same */
+
+static char rcolors[] = { 19, 20, 22, 24, 25, 26, 27, 28, 29 };
+
+static int
+color_of (char *name)
+{
+	int i = 0, sum = 0;
+
+	while (name[i])
+		sum += name[i++];
+	sum %= sizeof (rcolors) / sizeof (char);
+	return rcolors[sum];
+}
+
+
 /* called by EMIT_SIGNAL macro */
 
 void
@@ -1408,21 +1533,21 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 {
 	char *word[PDIWORDS];
 	int i;
+	unsigned int stripcolor_args = (prefs.stripcolor ? 0xFFFFFFFF : 0);
+	char tbuf[NICKLEN + 4];
 
-	if (!a)
-		a = "\000";
-	if (!b)
-		b = "\000";
-	if (!c)
-		c = "\000";
-	if (!d)
-		d = "\000";
+	if (prefs.colorednicks && (index == XP_TE_CHANACTION || index == XP_TE_CHANMSG))
+	{
+		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (a), a);
+		a = tbuf;
+		stripcolor_args &= ~ARG_FLAG(1);	/* don't strip color from this argument */
+	}
 
 	word[0] = te[index].name;
-	word[1] = a;
-	word[2] = b;
-	word[3] = c;
-	word[4] = d;
+	word[1] = (a ? a : "\000");
+	word[2] = (b ? b : "\000");
+	word[3] = (c ? c : "\000");
+	word[4] = (d ? d : "\000");
 	for (i = 5; i < PDIWORDS; i++)
 		word[i] = "\000";
 
@@ -1433,7 +1558,7 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 
 	/* If a plugin's callback executes "/close", 'sess' may be invalid */
 	if (is_session (sess))
-		display_event (sess, index, word);
+		display_event (sess, index, word, stripcolor_args);
 }
 
 int
