@@ -102,6 +102,8 @@ static void     drop_send                             (ConversationPanel      *p
 static void     drop_paste                            (ConversationPanel      *panel);
 static void     send_file                             (gpointer                file,
                                                        gpointer                user_data);
+static void     on_default_copy_activate              (GtkAction              *action,
+                                                       ConversationPanel      *panel);
 static gboolean uri_is_text                           (gchar *uri);
 static gboolean check_file_size                       (gchar *uri);
 
@@ -143,6 +145,10 @@ static GtkActionEntry dnd_actions[] = {
 	{ "DropCancel",        NULL, N_("_Cancel"),              NULL, NULL, G_CALLBACK (drop_cancel) },
 };
 
+static GtkActionEntry default_actions[] = {
+	{ "DefaultCopy",     GTK_STOCK_COPY, N_("_Copy"),        NULL, NULL, G_CALLBACK (on_default_copy_activate) },
+};
+
 #define DROP_FILE_PASTE_MAX_SIZE 1024
 enum
 {
@@ -212,6 +218,7 @@ conversation_panel_init (ConversationPanel *panel)
 	gtk_action_group_add_actions (action_group, url_actions,   G_N_ELEMENTS (url_actions),   panel);
 	gtk_action_group_add_actions (action_group, email_actions, G_N_ELEMENTS (email_actions), panel);
 	gtk_action_group_add_actions (action_group, dnd_actions,   G_N_ELEMENTS (dnd_actions),   panel);
+	gtk_action_group_add_actions (action_group, default_actions, G_N_ELEMENTS (default_actions),   panel);
 	gtk_ui_manager_insert_action_group (gui.manager, action_group, 0);
 	g_object_unref (action_group);
 
@@ -311,14 +318,28 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 		case WORD_NICK:
 			{
 				char *command;
-				command = g_strdup_printf ("QUERY %s", word);
-				handle_command (gui.current_session, command, 1);
-				g_free (command);
+				struct session *sess;
+
+				sess = find_dialog (gui.current_session->server, word);
+				if (sess)
+					navigation_tree_select_session (gui.server_tree, sess);
+				else {
+					command = g_strdup_printf ("QUERY %s", word);
+					handle_command (gui.current_session, command, 1);
+					g_free (command);
+				}
 			}
 			break;
 		case WORD_CHANNEL:
 			{
 				char *command;
+				struct session *sess;
+
+				sess = find_channel (gui.current_session->server, word);
+				if (sess)
+					navigation_tree_select_session (gui.server_tree, sess);
+				/* Having the channel opened doesn't mean that we're still on it
+				 * (if channel was left) */
 				command = g_strdup_printf ("JOIN %s", word);
 				handle_command (gui.current_session, command, 1);
 				g_free (command);
@@ -329,7 +350,12 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 	if (event->button == 3) {
 		switch (conversation_panel_check_word (xtext, word, strlen (word))) {
 		case 0:
-			/* FIXME: show default? context menu */
+			{
+				GtkWidget *menu;
+				menu = gtk_ui_manager_get_widget (gui.manager, "/DefaultPopup");
+				/* FIXME: we should not display the copy action if no text is selected */
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+			}
 			break;
 		case WORD_URL:
 		case WORD_HOST:
@@ -383,13 +409,17 @@ static void
 conversation_panel_set_font (ConversationPanel *panel)
 {
 	GConfClient *client;
-	gchar       *font;
+	gchar       *font = NULL;
 
 	client = gconf_client_get_default ();
-	if (gconf_client_get_bool(client, "/apps/xchat/main_window/use_sys_fonts", NULL))
-		font = gconf_client_get_string (client, "/desktop/gnome/interface/monospace_font_name", NULL);
-	else
+	if (!gconf_client_get_bool(client, "/apps/xchat/main_window/use_sys_fonts", NULL))
 		font = gconf_client_get_string (client, "/apps/xchat/main_window/font", NULL);
+
+	/* Either use_sys_fonts==TRUE, or there is no current font preference.
+	 * In both cases we try the GNOME monospace font. */
+	if (font == NULL)
+		font = gconf_client_get_string (client, "/desktop/gnome/interface/monospace_font_name", NULL);
+
 	g_object_unref (client);
 
 	if (font == NULL)
@@ -552,7 +582,7 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 	case TARGET_UTF8_STRING:
 	case TARGET_TEXT_PLAIN:
 	{
-		gchar *txt;
+		guchar *txt;
 
 		txt = gtk_selection_data_get_text (selection_data);
 		if (gui.current_session != NULL)
@@ -563,7 +593,8 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 	}
 	case TARGET_URI_LIST:
 	{
-		gchar *uri_list, **uris;
+		guchar *uri_list;
+		gchar **uris;
 		gint nb_uri;
 
 		if ((panel->priv->current->type != SESS_CHANNEL) &&
@@ -686,6 +717,12 @@ send_file (gpointer file, gpointer user_data)
 	}
 }
 
+static void
+on_default_copy_activate (GtkAction *action, ConversationPanel *panel)
+{
+	gtk_editable_copy_clipboard (GTK_EDITABLE (gui.text_entry));
+}
+
 static gboolean
 uri_is_text (gchar *uri)
 {
